/**
 * Back 2 Browser Bytecode Translator
 * Copyright (C) 2012 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://opensource.org/licenses/GPL-2.0.
 */
package net.java.html.canvas.spi;

import net.java.html.canvas.Dimension;
import net.java.html.canvas.Image;
import net.java.html.canvas.Style;

/**
 * Provider API for Canvas. Implement this to add support for your platform.
 *
 * @author antonepple
 */
public interface GraphicsEnvironment {

    public void arc(double centerX,
            double centerY,
            double startAngle,
            double radius,
            double endAngle,
            boolean ccw);

    public void arcTo(double x1,
            double y1,
            double x2,
            double y2,
            double r);

    public boolean isPointInPath(double x, double y);

    public void fill();

    public void stroke();

    public void beginPath();

    public void closePath();

    public void clip();

    public void moveTo(double x, double y);

    public void lineTo(double x, double y);

    public void quadraticCurveTo(double cpx, double cpy, double x, double y);

    public void bezierCurveTo(double cp1x, double cp1y, double cp2x, double cp2y, double x, double y);

    public void fillRect(double x, double y, double width, double height);

    public void strokeRect(double x, double y, double width, double height);

    public void clearRect(double x, double y, double width, double height);

    public void rect(double x, double y, double width, double height);

    public void save();

    public void restore();

    public void rotate(double angle);

    public void transform(double a, double b, double c, double d, double e, double f);

    public void setTransform(double a, double b, double c, double d, double e, double f);

    public void translate(double x, double y);

    public void scale(double x, double y);

    public Object drawImage(Image image, double x, double y, Object nativeImage);

    public Object drawImage(Image image, double x, double y, double width, double height, Object nativeImage);

    public Object drawImage(Image image, double sx, double sy, double sWidth, double sHeight, double x, double y, double width, double height, Object nativeImage);

    public int getWidth(Image image, Object nativeImage);

    public int getHeight(Image image, Object nativeImage);

    /**
     * When implementing you can return an Object of your choice to enable
     * caching. Returning null means no caching. When caching is enabled, and
     * the cache hasn't been invalidated, the Object you returned will be passed
     * as a parameter.
     *
     * @param style The style object you should use to create your native style
     * @param nativeStyle your native object if cached, null otherwise
     * @return return native Object for caching
     *
     */
    public Object setFillStyle(Style style, Object nativeStyle);

    /**
     * When implementing you can return an Object of your choice to enable
     * caching. Returning null means no caching. When caching is enabled, and
     * the cache hasn't been invalidated, the Object you returned will be passed
     * as a parameter.
     *
     * @param style The style object you should use to create your native style
     * @param nativeStyle your native object if cached, null otherwise
     * @return return native Object for caching
     *
     */
    public Object setStrokeStyle(Style style, Object nativeStyle);

//    public void setShadowColor(String color);
//
//    public void setShadowBlur(double blur);
//
//    public void setShadowOffsetX(double x);
//
//    public void setShadowOffsetY(double y);
//    
//    public String getShadowColor();
//
//    public double getShadowBlur();
//
//    public double getShadowOffsetX();
//
//    public double getShadowOffsetY();
    /**
     * Gets the current stroke line cap.
     *
     * @return {@code StrokeLineCap} with a value of Butt, Round, or Square.
     */
    public String getLineCap();

    /**
     * Sets the current stroke line cap.
     *
     * @param style a value of Butt, Round, or Square.
     */
    public void setLineCap(String style);

    /**
     * Gets the current stroke line join.
     *
     * @return a value of Miter, Bevel, or Round.
     */
    public String getLineJoin();

    /**
     * Sets the current stroke line join.
     *
     * @param style with a value of Miter, Bevel, or Round.
     */
    public void setLineJoin(String style);

    /**
     * Gets the current line width.
     *
     * @return value between 0 and infinity.
     */
    public double getLineWidth();

    /**
     * Sets the current line width.
     *
     * @param width value in the range {0-positive infinity}, with any other
     * value being ignored and leaving the value unchanged.
     */
    public void setLineWidth(double width);

    /**
     * Gets the current miter limit. v
     *
     * @return the miter limit value in the range {@code 0.0-positive infinity}
     */
    public double getMiterLimit();

    /**
     * Sets the current miter limit.
     *
     * @param limit miter limit value between 0 and positive infinity with any
     * other value being ignored and leaving the value unchanged.
     */
    public void setMiterLimit(double limit);

    /**
     * Gets the current Font.
     *
     * @return the Font
     */
    public String getFont();

    /**
     * Sets the current Font.
     *
     */
    public void setFont(String font);

    /**
     * Gets the current {@code TextAlignment}.
     *
     * @return TextAlignment with values of Left, Center, Right, or Justify.
     */
    public String getTextAlign();

    /**
     * Defines horizontal text alignment, relative to the text origin.
     *
     * @param textAlign with values of Left, Center, Right.
     */
    public void setTextAlign(String textAlign);

    /**
     * Sets the current Text Baseline.
     *
     * @param baseline with values of Top, Center, Baseline, or Bottom
     */
    public String getTextBaseline();

    /**
     * Sets the current Text Baseline.
     *
     * @param baseline with values of Top, Center, Baseline, or Bottom
     */
    public void setTextBaseline(String baseline);

    /**
     * Fills the given string of text at position x, y (0,0 at top left) with
     * the current fill paint attribute.
     *
     * @param text the string of text.
     * @param x position on the x axis.
     * @param y position on the y axis.
     */
    public void fillText(String text, double x, double y);

    /**
     * Fills text and includes a maximum width of the string.
     *
     * If the width of the text extends past max width, then it will be sized to
     * fit.
     *
     * @param text the string of text.
     * @param x position on the x axis.
     * @param y position on the y axis.
     * @param maxWidth maximum width the text string can have.
     */
    public void fillText(String text, double x, double y, double maxWidth);

    /**
     * The Dimension of this text using the current Font settings
     *
     * @param text
     * @return the Dimension of this text using the current Font settings
     */
    public Dimension measureText(String text);

    /**
     * draws the given string of text at position x, y (0,0 at top left) with
     * the current stroke paint attribute.
     *
     * @param text the string of text.
     * @param x position on the x axis.
     * @param y position on the y axis.
     */
    public void strokeText(String text, double x, double y);

    /**
     * Draws text with stroke paint and includes a maximum width of the string.
     *
     * If the width of the text extends past max width, then it will be sized to
     * fit.
     *
     * @param text the string of text.
     * @param x position on the x axis.
     * @param y position on the y axis.
     * @param maxWidth maximum width the text string can have.
     */
    public void strokeText(String text, double x, double y, double maxWidth);

//    public ImageData createPixelMap(double x, double y);
//
//    public ImageData createPixelMap(ImageData imageData);
//
//    public ImageData getPixelMap(double x, double y, double width, double height);
//
//    public void putPixelMap(ImageData imageData, double x, double y);
//
//    public void putPixelMap(ImageData imageData, double x, double y, double dirtyx, double dirtyy, double dirtywidth, double dirtyheight);
    /**
     * Sets the global alpha of the current state.
     *
     * @param alpha value in the range {@code 0.0-1.0}. The value is clamped if
     * it is out of range.
     */
    public void setGlobalAlpha(double alpha);

    public double getGlobalAlpha();

    public void setGlobalCompositeOperation(String operation);

    public String getGlobalCompositeOperation();

    public int getHeight();

    public int getWidth();

//    public void setHeight(int height);
//
//    public void setWidth(int width);

    public Object mergeImages(Image a, Image b, Object cachedA, Object cachedB);
}
